/*
linphone
Copyright (C) 2000  Simon MORLAT (simon.morlat@free.fr)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "linphone_config.h"
#include "linphonecore.h"
#include <mediastream.h>
#include <osipua.h>
#include "sdphandler.h"
#include "misc.h"

#include <telephonyevents.h>
#include <utils.h>
#include "osipuacb.h"

#include <net/if.h>
#include <sys/ioctl.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include <netdb.h>


#define MAX_IF 20
const double bandwidths[5]=
{
	28800.0,	/*28k modem*/
	28800.0, /*56k modem*/
	64000.0, /*64k modem (numeris)*/
	128000.0,	/*adsl or cable*/
	10e6, /* ethernet */
};/* carefull: these are the upstream bandwidth !!!*/


/* these are useful messages for the status */
gchar *ready;
gchar *end;
gchar *contacting;
gchar *connected;
gchar *cancel;
gchar *contacted;


void lc_callback_obj_init(LCCallbackObj *obj,LinphoneCoreCbFunc func,gpointer ud)
{
  obj->_func=func;
  obj->_user_data=ud;
}

gint lc_callback_obj_invoke(LCCallbackObj *obj, LinphoneCore *lc){
  if (obj->_func!=NULL) obj->_func(lc,obj->_user_data);
  return 0;
}

LinphoneDialogParams * linphone_dialog_params_new()
{
	return g_new0(LinphoneDialogParams,1);
}

void linphone_dialog_params_destroy(LinphoneDialogParams *obj)
{
	if (obj->profile!=&av_profile) rtp_profile_destroy(obj->profile);
	g_free(obj);
}


gboolean
linphone_core_init (LinphoneCore * lc,LinphoneCoreVTable *vtable, gpointer userdata)
{
	int err;

	//TRACE_INITIALIZE(TRACE_LEVEL5,stdout);
	osipua_init();

	ortp_init();
	ortp_set_debug_file("oRTP",NULL);
	rtp_profile_set_payload(&av_profile,115,&lpc1015);
	rtp_profile_set_payload(&av_profile,116,&truespeech);
	rtp_profile_set_payload(&av_profile,101,&telephone_event);
	ms_init(); 

	memset (lc, 0, sizeof (LinphoneCore));
	lc->data=userdata;

	ready=_("Ready.");
	end=_("Communication ended.");
	contacting= _("Contacting ");
	contacted=_("is calling you.");
	connected=_("Connected.");
	cancel=_("Call cancelled.");
		
	memcpy(&lc->vtable,vtable,sizeof(LinphoneCoreVTable));

	lc->ua=osip_ua_new();
  	err=osip_ua_signal_connect(lc->ua,"INVITE_ACCEPTED",invite_accepted_cb);
  	err=osip_ua_signal_connect(lc->ua,"BYE",bye_cb);
  	err=osip_ua_signal_connect(lc->ua,"FAILLURE",faillure_cb);
  	err=osip_ua_signal_connect(lc->ua,"INVITE",invite_cb);
  	err=osip_ua_signal_connect(lc->ua,"INFORMATIVE",informative_cb);
  	lc->ua->data=(void*)lc;
	
	/* create a sdp handler */
	lc->sdph=sdp_handler_new();
	sdp_handler_set_write_offer_fcn(SDP_HANDLER(lc->sdph),set_audio_offer,set_video_offer);
	sdp_handler_set_accept_offer_fcn(SDP_HANDLER(lc->sdph),accept_audio_offer,accept_video_offer);
	sdp_handler_set_read_answer_fcn(SDP_HANDLER(lc->sdph),read_audio_answer,read_video_answer);
	
	/* add the sdp handler to the ua */
	osip_ua_add_body_handler(lc->ua,lc->sdph);

	/* temporarily init some config structures here. */
	/* they should eventually go to Gaim's pref... */
	lc->auto_connect = 1;
	
#if 0
	/* sound configuation */
	linphone_core_set_sound_device(lc,0);
	linphone_core_set_play_level(lc,80);
	linphone_core_set_rec_level(lc,80);
	linphone_core_set_sound_source(lc,'m');
	/* linphone_core_set_ringback(lc,0); */
	lc->sound_conf.autokill=1;
	check_sound_device(lc);
#endif

	/* registrar configuration
	// linphone_core_set_registrar_addr(lc,tmpstr,tmpstr2);
	//linphone_core_set_registrar_passwd(lc,tmpstr);
	//linphone_core_set_registrar_usage(lc,tmp1,tmp2,tmp3);
	*/

	/* network configuration 
	//linphone_core_set_sip_port(lc,5060);
	// linphone_core_set_primary_contact(lc,g_strdup_printf("sip:%s@%s",username,hostname));
	//linphone_core_set_audio_port(lc,7078);
	//linphone_core_set_video_port(lc,9078);
	*/
	linphone_core_set_connection_type(lc, CONNECTION_TYPE_ETHERNET);

	/* codecs configuration
	//do we really need it??
	//pt=payload_type_new();
	//pt->mime_type="PCMA";
	//pt->clock_rate=8000;
	//pt->flags |= PAYLOAD_TYPE_ENABLED;
	*/
	linphone_core_setup_local_rtp_profile(lc);

	/* jitt compensation */
	linphone_core_set_audio_jittcomp(lc,60);
	linphone_core_set_video_jittcomp(lc,60);

	lc->in_main_thread=1;
	lc->lock=g_mutex_new();

	return TRUE;
}

static void free_payload(gpointer data, gpointer user_data)
{
	PayloadType * pt = data;
	pt->user_data = NULL;	/* detach it from mediastreamer. */
}

void linphone_core_uninit(LinphoneCore *lc)
{
	/* unregister if necessary*/
	if (lc->sip_conf.reg_conf.use_registrar){
		do_registration(lc,0);
		sleep(1);
	}

	if (lc->sip_conf.contact) g_free(lc->sip_conf.contact);
	if (lc->sip_conf.username) g_free(lc->sip_conf.username);
	if (lc->sip_conf.hostname) g_free(lc->sip_conf.hostname);
	if (lc->sip_conf.reg_conf.registrar) g_free(lc->sip_conf.reg_conf.registrar);
	if (lc->sip_conf.reg_conf.passwd) g_free(lc->sip_conf.reg_conf.passwd);
	if (lc->sip_conf.reg_conf.addr_of_rec) g_free(lc->sip_conf.reg_conf.addr_of_rec);
	if (lc->sip_conf.reg_conf.timer) g_timer_destroy(lc->sip_conf.reg_conf.timer);

#if 0
	if (lc->sound_conf.sndcard) g_free(lc->sound_conf.sndcard);
	if (lc->sound_conf.local_ring) g_free(lc->sound_conf.local_ring);
	if (lc->sound_conf.remote_ring) g_free(lc->sound_conf.remote_ring);
#endif	

	/* statically allocated, can't be freed */
	if (lc->codecs_conf.audio_codecs)
	{
			g_list_foreach(lc->codecs_conf.audio_codecs,(GFunc)free_payload,NULL);
			g_list_free(lc->codecs_conf.audio_codecs);
	}
	if (lc->codecs_conf.video_codecs)
	{
			g_list_foreach(lc->codecs_conf.video_codecs,(GFunc)free_payload, NULL);
			g_list_free(lc->codecs_conf.video_codecs);
	}

	if (lc->net_conf.nat_address) g_free(lc->net_conf.nat_address);

	ortp_exit();
	osipua_exit();

	if (lc->ua) osip_ua_destroy(lc->ua);
	if (lc->sdph) sdp_handler_destroy(lc->sdph);
}

LinphoneCore *linphone_core_new(LinphoneCoreVTable *vtable,
						gchar *config_path, gpointer userdata)
{
	LinphoneCore *core=g_new0(LinphoneCore,1);
	linphone_core_init(core,vtable,userdata);
	return core;
}



/************************************
* API functions
*************************************/
/* network connection APIs */
void linphone_core_set_connection_type(LinphoneCore *lc, int type)
{
	lc->net_conf.con_type=type;
	lc->net_conf.bandwidth=bandwidths[type];
	linphone_core_check_codecs_for_bandwidth(lc);
}

int linphone_core_get_connection_type(LinphoneCore *lc)
{
	return lc->net_conf.con_type;
}

int linphone_core_get_sip_port(LinphoneCore *lc)
{
	return lc->sip_conf.sip_port;
}

void linphone_core_set_sip_port(LinphoneCore *lc,int port)
{
	lc->sip_conf.sip_port=port;
	linphone_core_update_contact_info(lc);
}

void linphone_core_set_nat_address(LinphoneCore *lc, gchar *addr, gboolean use)
{
	gchar *tmp=NULL;
	gint err;
	struct addrinfo hints,*res;

	if (addr!=NULL && use){
		memset(&hints,0,sizeof(struct addrinfo));
		hints.ai_family=PF_UNSPEC;
		hints.ai_socktype = SOCK_DGRAM;
		err=getaddrinfo(addr,NULL,&hints,&res);
		if (err==0){
			tmp=g_strdup(addr);
			freeaddrinfo(res);
		}else {
			g_warning("Invalid nat address %s",addr);
			return;
		}
	}
	if (lc->net_conf.nat_address!=NULL){
		g_free(lc->net_conf.nat_address);
	}
	lc->net_conf.nat_address=tmp;
	if (use && (tmp!=NULL)){
		gchar *ct;
		/* change OsipUA object main contact */
		if (lc->sip_conf.sip_port==5060){
			ct=g_strdup_printf("sip:%s@%s",lc->sip_conf.username,tmp);
		}else{
			ct=g_strdup_printf("sip:%s@%s:%i",lc->sip_conf.username,tmp,
							lc->sip_conf.sip_port);
		}
		osip_ua_set_contact(lc->ua,ct);
		g_free(ct);
		lc->net_conf.use_nat=TRUE;
	}
	else {
		lc->net_conf.use_nat=FALSE;
		linphone_core_update_contact_info(lc);
	}
}

gchar *linphone_core_get_nat_address(LinphoneCore *lc, gboolean *use)
{
	if (use!=NULL) *use=lc->net_conf.use_nat;
	if (lc->net_conf.nat_address!=NULL) return g_strdup(lc->net_conf.nat_address);
	else return NULL;
}


/* codec related APIs */
GList *linphone_core_get_audio_codecs(LinphoneCore *lc)
{
	return g_list_copy(lc->codecs_conf.audio_codecs);
}

GList *linphone_core_get_video_codecs(LinphoneCore *lc)
{
	return g_list_copy(lc->codecs_conf.video_codecs);
}

int linphone_core_set_audio_codecs(LinphoneCore *lc, GList *codecs)
{
	if (lc->codecs_conf.audio_codecs!=NULL) g_list_free(lc->codecs_conf.audio_codecs);
	lc->codecs_conf.audio_codecs=codecs;
	return 0;
}

int linphone_core_set_video_codecs(LinphoneCore *lc, GList *codecs)
{
	if (lc->codecs_conf.video_codecs!=NULL) g_list_free(lc->codecs_conf.video_codecs);
	lc->codecs_conf.video_codecs=codecs;
	return 0;
}

/* contact APIs */
int linphone_core_set_primary_contact(LinphoneCore *lc, gchar *contact)
{
	int error=0;
	from_t *ct;
	from_init(&ct);
	if ( from_parse(ct,contact) <0){
		g_warning("Bad contact url: %s", contact);
		error=1;
		goto end;
	}
	lc->sip_conf.username=g_strdup(ct->url->username);
	lc->sip_conf.hostname=g_strdup(ct->url->host);
	linphone_core_update_contact_info(lc);
	end:
		from_free(ct);
		sfree(ct);
	return error;
}

gchar *linphone_core_get_primary_contact(LinphoneCore *lc)
{
	return g_strdup(lc->sip_conf.contact);
}

from_t *linphone_core_get_primary_contact_parsed(LinphoneCore *lc)
{
	int err;
	from_t *contact;
	from_init(&contact);
	err=from_parse(contact,lc->sip_conf.contact);
	if (err<0) {
		from_free(contact);
		sfree(contact);
		return NULL;
	}
	return contact;
}

/* registrar related APIs */
char *linphone_core_get_registrar_addr(LinphoneCore *lc)
{
	return g_strdup(lc->sip_conf.reg_conf.registrar);
}

int linphone_core_set_registrar_addr(LinphoneCore *lc,char *url,char *addr_of_rec)
{
	url_t *surl=NULL,*surl2=NULL;
	int err=0;
	if (url==NULL || strlen(url)==0) return 0;
	url_init(&surl);
	err=url_parse(surl,url);
	if (err<0){
		g_warning("Bad proxy or registrar url:%s",url);
		goto end;
	}
	if (addr_of_rec!=NULL){
		url_init(&surl2);
		err=url_parse(surl2,addr_of_rec);
		if (err<0){
			g_warning("Bad address of record:%s",addr_of_rec);
			goto end;
		}
	}
	if (lc->sip_conf.reg_conf.registrar!=NULL){
		g_free(lc->sip_conf.reg_conf.registrar);
	}
	lc->sip_conf.reg_conf.registrar=g_strdup(url);
	
	if (addr_of_rec!=NULL){
		if (lc->sip_conf.reg_conf.addr_of_rec!=NULL){
			g_free(lc->sip_conf.reg_conf.addr_of_rec);
		}
		lc->sip_conf.reg_conf.addr_of_rec=g_strdup(addr_of_rec);
	}else{
		/* guess a valid address of record */
		if (surl->port!=NULL)
			lc->sip_conf.reg_conf.addr_of_rec=g_strdup_printf("sip:%s@%s:%s",
				lc->sip_conf.username,surl->host,surl->port);
		else lc->sip_conf.reg_conf.addr_of_rec=g_strdup_printf("sip:%s@%s",
				lc->sip_conf.username,surl->host);
	}
	end:
		url_free(surl);
		sfree(surl);
		if (surl2!=NULL){
			url_free(surl2);
			sfree(surl2);
		}
	return err;
}

char *linphone_core_get_addr_of_rec(LinphoneCore *lc)
{
	return g_strdup(lc->sip_conf.reg_conf.addr_of_rec);
}

char *linphone_core_get_registrar_passwd(LinphoneCore *lc)
{
	return g_strdup(lc->sip_conf.reg_conf.passwd);
}

void linphone_core_set_registrar_passwd(LinphoneCore *lc, gchar *passwd)
{
	gchar *tmp=NULL;
	if (passwd!=NULL){
		tmp=g_strdup(passwd);
	}
	if (lc->sip_conf.reg_conf.passwd!=NULL){
		g_free(lc->sip_conf.reg_conf.passwd);
	}
	lc->sip_conf.reg_conf.passwd=tmp;
}


int linphone_core_register(LinphoneCore *lc, char *url){
	if (url!=NULL){
		linphone_core_set_registrar_addr(lc,url,NULL);
	}
	do_registration(lc,1);
	return 0;
}

void linphone_core_get_registrar_usage(LinphoneCore *lc,gboolean *reg, gboolean *proxy, gint *expires)
{
	if (reg!=NULL) *reg= lc->sip_conf.reg_conf.use_registrar;
	if (proxy!=NULL) *proxy=lc->sip_conf.reg_conf.as_proxy;
	if (expires!=NULL) *expires=lc->sip_conf.reg_conf.expires;
}



void linphone_core_set_registrar_usage(LinphoneCore *lc, gboolean use, gboolean outbound_proxy,gint expires)
{
	if (lc->sip_conf.reg_conf.timer==NULL){
		GTimer *timer;
		timer=g_timer_new();
		g_timer_start(timer);
		lc->sip_conf.reg_conf.timer=timer;
	}
	if (expires<=0) lc->sip_conf.reg_conf.expires=15*60;
	else lc->sip_conf.reg_conf.expires=expires;
	if ((lc->sip_conf.reg_conf.registrar!=NULL) && (strlen(lc->sip_conf.reg_conf.registrar)!=0)){
		if (use){
			if (outbound_proxy)
				osip_ua_set_outbound_proxy(lc->ua,lc->sip_conf.reg_conf.registrar,lc->sip_conf.reg_conf.passwd);
			else osip_ua_set_outbound_proxy(lc->ua,NULL,NULL);
			do_registration(lc,1);
			/* TODO: setup a timeout to re-register after expires */
		}else{
			osip_ua_set_outbound_proxy(lc->ua,NULL,NULL);
			if (lc->sip_conf.reg_conf.use_registrar){
				/* the proxy is no more used, we need to unregister */
				do_registration(lc,0);
			}
		}
	}
	lc->sip_conf.reg_conf.use_registrar=use;
	lc->sip_conf.reg_conf.as_proxy=outbound_proxy;
}


/* audio related APIs */
int linphone_core_get_audio_jittcomp(LinphoneCore *lc)
{
	return lc->rtp_conf.audio_jitt_comp;
}

int linphone_core_get_audio_port(LinphoneCore *lc)
{
	return lc->rtp_conf.audio_rtp_port;
}

void linphone_core_set_audio_jittcomp(LinphoneCore *lc, int value)
{
	lc->rtp_conf.audio_jitt_comp=value;
}

void linphone_core_set_audio_port(LinphoneCore *lc, int port)
{
	lc->rtp_conf.audio_rtp_port=port;
}


/* Video related APIs */
int linphone_core_get_video_jittcomp(LinphoneCore *lc)
{
	return lc->rtp_conf.video_jitt_comp;
}

int linphone_core_get_video_port(LinphoneCore *lc)
{
	return lc->rtp_conf.video_rtp_port;
}

void linphone_core_set_video_jittcomp(LinphoneCore *lc, int value)
{
	lc->rtp_conf.video_jitt_comp=value;
}

void linphone_core_set_video_port(LinphoneCore *lc, int port)
{
	lc->rtp_conf.video_rtp_port=port;
}

void linphone_core_enable_video(LinphoneCore *lc, gboolean enable)
{
	lc->video_conf.enabled = (gboolean)enable;
}

gboolean linphone_core_video_enabled(LinphoneCore *lc)
{
	if(lc->video_conf.enabled != 0)
		return 1;
	else
		return 0;
}

gboolean sipurl_check(gchar *url)
{
	from_t *surl;
	int err,ret;
	if (url==NULL) return FALSE;
	
	from_init(&surl);
	err=from_parse(surl,url);
	if (err<0) ret=FALSE;
	else ret=TRUE;
	from_free(surl);
	sfree(surl);
	return ret;
}


/* dialog related APIs */
int linphone_core_invite(LinphoneCore *lc, char *url)
{
	gchar *barmsg;
	gint err;
	
	linphone_core_lock(lc);
	if (!sipurl_check(url) ){
		lc->vtable.display_warning(lc,_("Bad formuled sip address. A sip address looks like <sip:username@domainname>"));
		linphone_core_unlock(lc);
		return -1;
	}
	barmsg=g_malloc(strlen(url)+strlen(contacting)+2);
	sprintf(barmsg,"%s %s",contacting,url);
	lc->vtable.display_status(lc,barmsg);
	
	lc->call=osip_dialog_new(lc->ua);
	err=osip_dialog_invite(lc->call,url,"application/sdp");
	if (err!=0){
		g_warning("error in invite: %s (%i)\n",strerror(err),err);
		lc->vtable.display_status(lc,ready);
	}
	linphone_core_unlock(lc);
	return 0;
}

gboolean linphone_core_inc_invite_pending(LinphoneCore*lc){
	if (lc->call==NULL){
		return 0;
	}
	linphone_core_lock(lc);
	if (lc->call->status==DIALOG_INVITED){
		linphone_core_unlock(lc);
		return 1;
	}
	return 0;
}

int linphone_core_accept_dialog(LinphoneCore *lc, const char *url)
{
	LinphoneDialogParams *diaparams;
	StreamParams *audio_params;
	gint jitt_comp;
	
	if (lc->call==NULL){
		return -1;
	}
	linphone_core_lock(lc);
	if (lc->call->status!=DIALOG_INVITED){
		g_warning("No dialog to accept.");
		linphone_core_unlock(lc);
		return -1;
	}
	diaparams=(LinphoneDialogParams *)lc->call->data;
	audio_params=&diaparams->audio_params;
	
	/*stop ringing */
	if (lc->ringstream!=NULL) {
		ring_stop(lc->ringstream);
		lc->ringstream=NULL;
	}
	/* sends a 200 OK */
	osip_dialog_accept_invite(lc->call,lc->current_trn);
	lc->vtable.display_status(lc,connected);

	start_media_stream(lc);
#if 0
	if (try_open_dsp(lc)){
		/* adjust rtp jitter compensation. It must be at least the latency of the sound card */
		/* jitt_comp=MAX(lc->sound_conf.latency,lc->rtp_conf.audio_jitt_comp); */
		jitt_comp = lc->rtp_conf.audio_jitt_comp;
		lc->audiostream=audio_stream_start_with_sndcard(diaparams->profile,audio_params->localport,
							audio_params->remoteaddr,audio_params->remoteport,
							audio_params->pt,jitt_comp,lc->sound_conf.sndcard);
		/* start_audio_stream(lc, jitt_comp); */
	}

	if (linphone_core_video_enabled(lc))
	{
		/*
		StreamParams *video_params=&diaparams->video_params;
		lc->videostream = video_stream_start_full( video_params->localport, video_params->remoteaddr, video_params->remoteport,
			video_params->pt, 2000, NULL, NULL );
		*/
		start_video_stream(lc);
	}
#endif
  
	linphone_core_unlock(lc);
	return 0;
}

int linphone_core_terminate_dialog(LinphoneCore *lc, const char *url)
{
	OsipDialog *call=lc->call;
	LinphoneDialogParams *diaparams;
	if (call==NULL){
		return -1;
	}
	
	linphone_core_lock(lc);
	diaparams=(LinphoneDialogParams*)call->data;
	lc->call=NULL;
	
	if (call->status==DIALOG_INVITED )
    {
			/*stop ringing*/
		if (lc->ringstream!=NULL) {
			ring_stop(lc->ringstream);
			lc->ringstream=NULL;
			//restore_sound_daemons();
		}
		/* decline the invitation */
		osip_dialog_reject_invite(call);
		lc->vtable.display_status(lc,end);
    }
	else if (call->status==DIALOG_ESTABLISHED)
	{
		/* sends a bye, the call is terminated */
		#if 0
		if (lc->audiostream!=NULL) {
			audio_stream_stop(lc->audiostream);
			lc->audiostream=NULL;
			//restore_sound_daemons();
			osip_dialog_bye(call);
		}

		/*
		if (lc->videostream!=NULL) {
			video_stream_stop(lc->videostream);
			lc->videostream=NULL;
		}
		*/
		stop_video_stream();
		#endif
		
		stop_media_stream();
		osip_dialog_bye(call);
		lc->vtable.display_status(lc,end);
	}
	else if (call->status==DIALOG_INVITING)
	{
     /*sends a cancel or a bye ???*/
		if (lc->ringstream!=NULL) {
			ring_stop(lc->ringstream);
			lc->ringstream=NULL;
			//restore_sound_daemons();
		}
		osip_dialog_bye(call);
		lc->vtable.display_status(lc,cancel);
	}
	if (diaparams!=NULL) {
		call->data=NULL;
		linphone_dialog_params_destroy(diaparams);
	}
	linphone_core_unlock(lc);
	return 0;
}

void linphone_core_set_presence_info(LinphoneCore *lc,gint minutes_away,
													gchar *contact,
													int presence_mode)
{
	if (minutes_away>0) osip_ua_set_presence_delay(lc->ua,minutes_away*60);
	if (contact!=NULL) {
		osip_ua_set_presence_contact_url(lc->ua,contact);
	}
	osip_ua_set_presence_mode(lc->ua,presence_mode);
}

/* sound functions */
#if 0
gint linphone_core_get_play_level(LinphoneCore *lc)
{
	return lc->sound_conf.play_lev;
}

gint linphone_core_get_rec_level(LinphoneCore *lc){
	return lc->sound_conf.rec_lev;
}

void linphone_core_set_play_level(LinphoneCore *lc, gint level){
	lc->sound_conf.play_lev=level;
	g_return_if_fail(lc->sound_conf.sndcard!=NULL);
	snd_card_set_level(lc->sound_conf.sndcard,SND_CARD_LEVEL_OUTPUT,level);
}

void linphone_core_set_rec_level(LinphoneCore *lc, gint level)
{
	lc->sound_conf.rec_lev=level;
	g_return_if_fail(lc->sound_conf.sndcard!=NULL);
	snd_card_set_level(lc->sound_conf.sndcard,SND_CARD_LEVEL_INPUT,level);
}

gint linphone_core_set_sound_device(LinphoneCore *lc, gint devid)
{
	SndCard *sndcard;
	int tmp;
	if (devid<0){
		g_warning("Bad devid value: %i",devid);
		devid=0;
	}
	sndcard=snd_card_manager_get_card(snd_card_manager,devid);
	if (sndcard==NULL){
		g_warning("Sound card with index %i does not exist.",devid);
		devid=0;
	}
	lc->sound_conf.dev_id=devid;
	lc->sound_conf.sndcard=sndcard;
	tmp=test_audio_dev(devid);
	if (tmp>0) lc->sound_conf.latency=tmp;
	return 0;
}

gint linphone_core_set_sound_device_from_name(LinphoneCore *lc,gchar *name)
{
	SndCard *card;
	gint index;
	card=snd_card_manager_get_card_with_string(snd_card_manager,name,&index);
	if (card!=NULL){
		g_message("Setting card with id=%i",index);
		lc->sound_conf.dev_id=index;
		lc->sound_conf.sndcard=card;
		return 0;
	}
	return -1;
}

gint linphone_core_get_sound_device(LinphoneCore *lc)
{
	return lc->sound_conf.dev_id;
}

SndCardManager * linphone_core_get_card_manager(LinphoneCore *lc){
	return snd_card_manager;
}

gchar linphone_core_get_sound_source(LinphoneCore *lc)
{
	return lc->sound_conf.source;
}
void linphone_core_set_sound_source(LinphoneCore *lc, gchar source)
{
	lc->sound_conf.source=source;
	g_return_if_fail(lc->sound_conf.sndcard!=NULL);
	snd_card_set_rec_source(lc->sound_conf.sndcard,source);
}

void linphone_core_set_ring(LinphoneCore *lc,gchar *path){
	if (lc->sound_conf.local_ring!=0){
		g_free(lc->sound_conf.local_ring);
	}
	lc->sound_conf.local_ring=g_strdup(path);
}
gchar *linphone_core_get_ring(LinphoneCore *lc){
	return lc->sound_conf.local_ring;
}
#endif

/*
static void notify_end_of_ring(MSFilter *f,gint event, gpointer value,gpointer user_data){
  LinphoneCore *lc=(LinphoneCore*)user_data;
  lc->preview_finished=1;
}
*/

void linphone_core_send_dtmf(LinphoneCore *lc,gchar dtmf)
{
	if (lc->audiostream!=NULL){
		send_dtmf(lc->audiostream,dtmf);
	}
}


void linphone_core_destroy(LinphoneCore *lc){
	linphone_core_uninit(lc);
	g_free(lc);
}
