/***************************************************************************
                          videostreamer.c  -  description
                             -------------------
    begin                : Fri May 28 2004
    copyright            : (C) 2004 by Zhanglei Wang
    email                : filamoon@hotmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "linphone_config.h"
#include "linphonecore.h"
#include <mscodec.h>
#include <regctxt.h>
#include <stdlib.h>
#include <stdio.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <gdk/gdk.h>
#include "misc.h"
#include "sndutil.h"
#include <mediastream.h>
#include "interface.h"
#include <glib.h>

sound_config_t sound_conf;
struct _VideoStream *videostream = NULL;
struct _AudioStream *audiostream = NULL;

/* Global settings */
int localport[2];
char * remoteaddr;
int remoteport[2];
int pt[2];
gint jitt_comp[2];
int enable[2];

/****************************
*      init functions       *
****************************/

void sound_init()
{
	gaim_mediastreamer_set_sound_device(&sound_conf,0);
	gaim_mediastreamer_set_play_level(&sound_conf,80);
	gaim_mediastreamer_set_rec_level(&sound_conf,80);
	gaim_mediastreamer_set_sound_source(&sound_conf,'m');
	/* gaim_mediastreamer_set_ringback(lc,0); */
	sound_conf.autokill=1;
	check_sound_device(sound_conf.sndcard);
	
}

void sound_uninit()
{
	if (sound_conf.sndcard) g_free(sound_conf.sndcard);
	if (sound_conf.local_ring) g_free(sound_conf.local_ring);
	if (sound_conf.remote_ring) g_free(sound_conf.remote_ring);
	restore_sound_daemons();
}


/****************************
*      called by gui        *
****************************/
void rtpstreamer_stop_audio_streamer()
{
	if (!audiostream) return;
	audio_stream_stop(audiostream);
	audiostream = NULL;
}

void rtpstreamer_start_audio_streamer()
{
	if (audiostream) return;
	if (enable[0] && try_open_dsp(&sound_conf))
	{
		jitt_comp[0] = MAX(sound_conf.latency, jitt_comp[0]);
		audiostream=audio_stream_start_with_sndcard(&av_profile, localport[0],
				remoteaddr, remoteport[0],
				pt[0], jitt_comp[0], sound_conf.sndcard);
	}
}

void toggle_rgbtobgr()
{
	if (!videostream) return;
	if (videostream->localvo)
		ms_video_output_toggle_rgbtobgr(videostream->localvo);
	if (videostream->encoder)
		ms_AVencoder_r263_toggle_rgb_bgr(videostream->encoder);
}

void toggle_local_view()
{
  static gboolean local_view = TRUE;
	if ( !videostream || !videostream->localvo) return;
	local_view = !local_view;
	ms_video_output_toggle_pause(videostream->localvo);
	/*
	//ms_sync_lock(videostream->timer);
	if (local_view)
	{
			//ms_video_output_toggle_pause(videostream->localvo);
			ms_filter_add_link (videostream->dispatcher, videostream->localvo);
	}
	else
	{
			//ms_video_output_toggle_pause(videostream->localvo);
			//ms_filter_lock(MS_FILTER(videostream->localvo));
			ms_filter_remove_links (videostream->dispatcher, videostream->localvo);
			//ms_filter_unlock(MS_FILTER(videostream->localvo));
	}
	//ms_sync_update(videostream->timer);
	//ms_sync_unlock(videostream->timer);
	*/
}

guint rtpstreamer_get_hue()
{
	g_return_if_fail(videostream != NULL);
	return ms_v4l_get_hue(videostream->source);
}

guint rtpstreamer_get_contrast()
{
	g_return_if_fail(videostream != NULL);
	return ms_v4l_get_contrast(videostream->source);
}

guint rtpstreamer_get_brightness()
{
	g_return_if_fail(videostream != NULL);
	return ms_v4l_get_brightness(videostream->source);
}

guint rtpstreamer_get_color()
{
	g_return_if_fail(videostream != NULL);
	return ms_v4l_get_color(videostream->source);
}

void rtpstreamer_set_hue(guint val)
{
	g_return_if_fail(videostream != NULL);
	ms_v4l_set_hue(videostream->source, val);
}

void rtpstreamer_set_contrast(guint val)
{
	g_return_if_fail(videostream != NULL);
	ms_v4l_set_contrast(videostream->source, val);
}

void rtpstreamer_set_brightness(guint val)
{
	g_return_if_fail(videostream != NULL);
	ms_v4l_set_brightness(videostream->source, val);
}

void rtpstreamer_set_color(guint val)
{
	g_return_if_fail(videostream != NULL);
	ms_v4l_set_color(videostream->source, val);
}

void rtpstreamer_set_speaker_level(guint val)
{
	g_return_if_fail(audiostream != NULL);
}

void rtpstreamer_set_mic_level(guint val)
{
	g_return_if_fail(audiostream != NULL);
}

guint rtpstreamer_get_speaker_level()
{
	g_return_if_fail(audiostream != NULL);
}

guint rtpstreamer_get_mic_level()
{
	g_return_if_fail(audiostream != NULL);
}


/********************************************************/
int main(int argc, char * argv[])
{
		GtkWidget * localptr, * remoteptr;
		
		if (argc != 12)
		{
			printf("gaim-mediastreamer Usage: gaim-mediastreamer \n"
					"\tremote_addr\n"
					"\tenable_voice localport_voice remoteport_voice pt_voice jitt_comp_voice\n"
					"\tenable_video localport_video remoteport_video pt_video jitt_comp_video\n" );
			_exit(-1);
		}
		
		g_warning("Linphone-vv: child process: g_thread_init...\n");
		g_thread_init(NULL);
		g_warning("Linphone-vv: child process: gdk_threads_init...\n");
		gdk_threads_init();
		g_warning("Linphone-vv: child process: gdk_init...\n");
		gtk_init(&argc, &argv);

		ortp_init();
		//ms_AVCodec_init();
		ms_init();
		sound_init();
		
		remoteaddr = argv[1];
		sscanf(argv[2], "%d", &enable[0]);
		sscanf(argv[3], "%d", &localport[0]);
		sscanf(argv[4], "%d", &remoteport[0]);
		sscanf(argv[5], "%d", &pt[0]);
		sscanf(argv[6], "%d", &jitt_comp[0]);
		sscanf(argv[7], "%d", &enable[1]);
		sscanf(argv[8], "%d", &localport[1]);
		sscanf(argv[9], "%d", &remoteport[1]);
		sscanf(argv[10], "%d", &pt[1]);
		sscanf(argv[11], "%d", &jitt_comp[1]);

		//printf("passing to rtp-streamer -> %s, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d", argv[1], enable[0], localport[0], remoteport[0], pt[0], jitt_comp[0], enable[1], localport[1], remoteport[1], pt[1], jitt_comp[1]);

		create_gui_window(&localptr, &remoteptr);
		
		if (enable[1])
		{
			g_warning("Linphone-vv: child process is starting video streams...\n");
			//ptr = create_gui_window();
			videostream = video_stream_start_full( localport[1], remoteaddr,
					remoteport[1],
					pt[1], jitt_comp[1], NULL, NULL, localptr, remoteptr );
		}
		else
			g_warning("Linphone-vv: child process: video not enabled.\n");

		/*
		if (enable[0] && try_open_dsp(&sound_conf))
		{
			jitt_comp[0] = MAX(sound_conf.latency, jitt_comp[0]);
			audiostream=audio_stream_start_with_sndcard(&av_profile, localport[0],
					remoteaddr, remoteport[0],
					pt[0], jitt_comp[0], sound_conf.sndcard);
		}
		*/
		rtpstreamer_start_audio_streamer();

		gtk_main();

		int sig;
		sigset_t sigs_to_catch;
		sigemptyset( &sigs_to_catch );
		sigaddset( &sigs_to_catch, SIGINT);
		sigaddset( &sigs_to_catch, SIGKILL);
		sigaddset( &sigs_to_catch, 30);
		sigwait( &sigs_to_catch, &sig );
		//usleep(20000);
		if (videostream) video_stream_stop(videostream);
		if (audiostream) audio_stream_stop(audiostream);
		g_warning("gaim-mediastreamer: exiting.\n");
		gtk_main_quit();
		_exit(0);
}
