
/*
linphone
Copyright (C) 2000  Simon MORLAT (simon.morlat@free.fr)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "linphone_config.h"
#include "linphonecore.h"
#include <mscodec.h>
#include <regctxt.h>
#include <stdlib.h>
#include <stdio.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#ifdef INET6
#include <sys/socket.h>
#include <netdb.h>
#endif

#include "misc.h"
#include <sys/wait.h>
#include <gdk/gdk.h>


/* send a REGISTER request to the default registrar*/
/* if doit is 0, then unregistration is done (expires is set to zero) */
gint do_registration(LinphoneCore *lc,gboolean doit)
{
	static RegistrationCtxt *regctx=NULL;
	
	if (lc->sip_conf.reg_conf.registrar==NULL) return FALSE;
	{
		OsipDialog *call=osip_dialog_new(lc->ua);
		if (regctx!=NULL) registration_ctxt_destroy(regctx);
		regctx=registration_ctxt_new();		
		registration_ctxt_set_registrar(regctx,lc->sip_conf.reg_conf.registrar);
		registration_ctxt_set_password(regctx, lc->sip_conf.reg_conf.passwd);
		registration_ctxt_set_address_of_record(regctx, lc->sip_conf.reg_conf.addr_of_rec);
		if (doit) {
			lc->vtable.display_status(lc,_("Registering..."));
			registration_ctxt_set_expires(regctx,lc->sip_conf.reg_conf.expires);
		}else {
			registration_ctxt_set_expires(regctx,0);
			//lc->vtable.display_status(lc,_("Unregistering..."));
		}
		
		g_timer_reset(lc->sip_conf.reg_conf.timer);
		osip_dialog_register(call,regctx);
		/* we don't want more from the dialog:*/
		osip_dialog_release(call);
		g_message("Registration sent.\n");
	}
	return TRUE;
}


void check_for_registration(LinphoneCore *lc)
{
	registrar_config_t *reg_conf;
	reg_conf=&lc->sip_conf.reg_conf;
	if (reg_conf->use_registrar){
		if (reg_conf->expires>0){
			gdouble time;
			time=g_timer_elapsed(reg_conf->timer,NULL);
			if ( ((int)time) > reg_conf->expires) do_registration(lc,1);
		}
	}
}

char *int2str(int number)
{
	gchar *numstr=g_malloc(10);
	snprintf(numstr,10,"%i",number);
	return numstr;
}




#define UDP_HDR_SZ 8
#define RTP_HDR_SZ 12
#define IP4_HDR_SZ 20   /*20 is the minimum, but there may be some options*/

gchar *payload_type_get_description(PayloadType *pt){
	if (pt->user_data!=NULL){
		MSFilterInfo *c=(MSFilterInfo*)pt->user_data;
		return c->description;
	}
	return NULL;
}	

void payload_type_set_enable(PayloadType *pt,gint value) 
{
	if ((value)!=0) payload_type_set_flag(pt,PAYLOAD_TYPE_ENABLED); \
	else payload_type_unset_flag(pt,PAYLOAD_TYPE_ENABLED); 
}
void payload_type_set_usable(PayloadType *pt,gint value) 
{
	if ((value)!=0) payload_type_set_flag(pt,PAYLOAD_TYPE_USABLE); \
	else payload_type_unset_flag(pt,PAYLOAD_TYPE_USABLE); 
}

/* return 0 if codec can be used with bandwidth, -1 else*/
int payload_type_check_usable(PayloadType *pt,double bandwidth)
{
	int ret=0;
	double codec_band;
	double npacket;
	double packet_size;
	
	switch (pt->type){
		case PAYLOAD_AUDIO_CONTINUOUS:
		case PAYLOAD_AUDIO_PACKETIZED:
			/* very approximative estimation... revisit*/
			npacket=50;
			packet_size=(double)(pt->normal_bitrate/(50*8))+UDP_HDR_SZ+RTP_HDR_SZ+IP4_HDR_SZ;
			codec_band=packet_size*8.0*npacket;
			ret=(codec_band<bandwidth);
			//g_message("Payload %s: %g",pt->mime_type,codec_band);
			break;
		case PAYLOAD_VIDEO:
			ret=-1;
	}
	
	
	return ret;
}

void linphone_core_check_codecs_for_bandwidth(LinphoneCore *lc)
{
	GList *elem;
	PayloadType *pt;
	
	elem=lc->codecs_conf.audio_codecs;
	while(elem!=NULL)
	{
		pt=(PayloadType*)elem->data;
		if (strcmp("speex",pt->mime_type)==0 && pt->clock_rate==8000){
			if (lc->net_conf.bandwidth<bandwidths[CONNECTION_TYPE_NUMERIS]){
				pt->normal_bitrate=8000;
			}else pt->normal_bitrate=20000;
		}	
		payload_type_set_usable(pt,payload_type_check_usable(pt,lc->net_conf.bandwidth));
		
		elem=g_list_next(elem);
	}

  /* added by Wang Zhanglei */
	elem=lc->codecs_conf.video_codecs;
	while(elem!=NULL)
	{
		pt=(PayloadType*)elem->data;
		payload_type_set_usable(pt,payload_type_check_usable(pt,lc->net_conf.bandwidth));
		elem=g_list_next(elem);
	}
  
}

GList *fix_codec_list(RtpProfile *prof, GList *conflist)
{
	GList *elem;
	GList *newlist=NULL;
	MSCodecInfo *codec;
	PayloadType *payload = NULL,*confpayload = NULL;
	for (elem=conflist;elem!=NULL;elem=g_list_next(elem))
	{
		confpayload=(PayloadType*)elem->data;
			
		payload=rtp_profile_find_payload(prof,
								confpayload->mime_type,
								confpayload->clock_rate);
		if (payload!=NULL){
			/*check if mediastreamer supports this payload*/
			switch (payload->type){
				case PAYLOAD_AUDIO_CONTINUOUS:
				case PAYLOAD_AUDIO_PACKETIZED:
					codec=ms_audio_codec_info_get(payload->mime_type);
					break;
				case PAYLOAD_VIDEO:
					codec=ms_video_codec_info_get(payload->mime_type);
				break;
			default:
				g_error("Unsupported rtp media type.");
			}
			if (codec==NULL){
				g_warning("There is no plugin to handle %s",confpayload->mime_type);
			}
			else 
			{
				payload_type_set_user_data(payload,(gpointer)codec);
				payload_type_set_enable(payload,payload_type_enabled(confpayload));
				newlist=g_list_append(newlist,(gpointer)payload);
			}
		}
		else{
			g_warning("Cannot support %s/%i: does not exist.",confpayload->mime_type,
								confpayload->clock_rate);
		}
	}
	return newlist;
}


void linphone_core_setup_local_rtp_profile(LinphoneCore *lc)
{
	int i;
	GList *audiopt = NULL, *videopt = NULL;
	MSCodecInfo *codec = NULL;
	PayloadType *payload = NULL;
	
	lc->local_profile=&av_profile;

	/* first look at the list given by configuration file to see if 
	it is correct */
	if (lc->codecs_conf.audio_codecs)
		audiopt=fix_codec_list(lc->local_profile,lc->codecs_conf.audio_codecs);

	/* modified by Wang zhanglei */
// 	printf("\n\n\nlalalalla\n\n\n");
	if (lc->codecs_conf.video_codecs)
	{
		//printf("\n\n\nlalalalla\n\n\n");
		videopt=fix_codec_list(lc->local_profile,lc->codecs_conf.video_codecs);
	}
	/* end of modification */
	
	/* now find and add payloads that are not listed in the configuration
	codec list */
	for (i=0;i<127;i++)
	{
		//printf("going %d\n", i);
		payload=rtp_profile_get_payload(lc->local_profile,i);
		if (payload!=NULL){
			//printf("payload is diff null\n");
			if (payload_type_get_user_data(payload)!=NULL) continue;
			//printf("passed the other\n");
			/* find a mediastreamer codec for this payload type */
			g_warning("Linphone plugin", "Found a payload %s in the local profile.\n", payload->mime_type);
			//printf("Linphone plugin Found a payload %s in the local profile.\n", payload->mime_type);
			switch (payload->type){
				case PAYLOAD_AUDIO_CONTINUOUS:
				case PAYLOAD_AUDIO_PACKETIZED:
					printf("got aud\n");
					codec=ms_audio_codec_info_get(payload->mime_type);
					if (codec!=NULL){
						g_warning("Linphone plugin","Adding new codec %s/%i\n",payload->mime_type,payload->clock_rate);
						//printf("Linphone plugin Adding new codec %s/%i\n",payload->mime_type,payload->clock_rate);
						payload_type_set_enable(payload,1);
						payload_type_set_user_data(payload,(gpointer)codec);
						audiopt=g_list_append(audiopt,(gpointer)payload);
					}
					break;
				case PAYLOAD_VIDEO:
					//printf("got vid\n");
					/* if (!linphone_core_video_enabled(lc)) break;   //added by Wang Zhanglei. */
					codec=ms_video_codec_info_get(payload->mime_type);
					if (codec!=NULL){
						//printf("added that payload as a vid\n");
						payload_type_set_enable(payload,1);
						payload_type_set_user_data(payload,(gpointer)codec);
						videopt=g_list_append(videopt,(gpointer)payload);
					}
					break;
				default:
					g_error("Unsupported rtp media type.");
			}
		}
	}

	/* modified by Wang zhanglei */
	g_list_foreach(lc->codecs_conf.audio_codecs,(GFunc)payload_type_destroy,NULL);
	g_list_free(lc->codecs_conf.audio_codecs);
	/* set the fixed lists instead:*/
	lc->codecs_conf.audio_codecs=audiopt;

	/* if (linphone_core_video_enabled(lc)) */
	/* { */
		g_list_foreach(lc->codecs_conf.video_codecs,(GFunc)payload_type_destroy,NULL);
		g_list_free(lc->codecs_conf.video_codecs);
	 	lc->codecs_conf.video_codecs=videopt;
	/* } */
	/* end of modification */

	linphone_core_check_codecs_for_bandwidth(lc);
}

int from_2char_without_params(from_t *from,char **str)
{
	gchar *tmp=NULL;
	*str=NULL;
	if (from->displayname!=NULL){
		if (from->url->username!=NULL){
			tmp=g_strdup_printf("%s <sip:%s@%s>",from->displayname,from->url->username,
								from->url->host);
		}else{
			tmp=g_strdup_printf("%s <sip:%s>",from->displayname,
								from->url->host);
		}
	}else{
		if (from->url->username!=NULL){
			tmp=g_strdup_printf("sip:%s@%s",from->url->username,from->url->host);
		}else{
			tmp=g_strdup_printf("sip:%s",from->url->host);
		}
		
	}
	g_return_val_if_fail(tmp!=NULL,-1);
	*str=sgetcopy(tmp);
	g_free(tmp);
	return 0;
}

int linphone_core_update_contact_info(LinphoneCore *lc)
{
        char *prim_contact;
        if (lc->sip_conf.sip_port==5060)
                prim_contact=g_strdup_printf("sip:%s@0.0.0.0",lc->sip_conf.username);
        else prim_contact=g_strdup_printf("sip:%s@0.0.0.0:%i",lc->sip_conf.username,lc->sip_conf.sip_port);
        if (lc->net_conf.use_nat) linphone_core_set_nat_address(lc,lc->net_conf.nat_address,1);
        else osip_ua_set_contact(lc->ua,prim_contact);
        g_free(prim_contact);
        /*linphone_core_set_alias(lc);*/

        return 1;
}


/*
void
start_audio_stream(LinphoneCore *lc, int jitt_comp)
{
}

void
stop_audio_stream()
{
}
*/

static int child_out[2], child_in[2];			/* communication pipes */
static pid_t child_pid = 0;

void
start_media_stream(LinphoneCore *lc)
{
	g_warning("Linphone plugin", "father process: trying to open media streams.\n");
	int i;
	
	if (child_pid != 0)
	{
			g_warning("Linphone plugin", "father process: a child process is running.\n");
			return;
	}
	
	LinphoneDialogParams *diaparams=(LinphoneDialogParams *)lc->call->data;
	StreamParams *video_params=&diaparams->video_params;
	StreamParams *audio_params=&diaparams->audio_params;
	pid_t pid;
  
	if(pipe(child_out) || pipe(child_in)) {
		g_warning("Linphone plugin",
				   "Could not create pipes: %s\n", strerror(errno));
		return;
	}
		
	pid = fork();

	if (pid==-1)
	{
			g_warning("Linphone plugin", "can't fork a child process.\n");
			close(child_out[0]);
			close(child_out[1]);
			close(child_in[1]);
			close(child_in[0]);
			return;
	}

	if (pid==0)
	{
		g_warning("Linphone plugin", "New child process, pid=%d!\n", child_pid = getpid());
		//write(child_out[1], &child_pid, sizeof(pid_t));
		close(child_out[0]);
		close(child_in[1]);

		char argv[12][128];
		char * argvv[13];
		for (i=0; i<12; i++)
			argvv[i] = argv[i];
		argvv[12] = NULL;
		int enable_video = (int)linphone_core_video_enabled(lc);

		//printf("enable video is : %d port are : %d %d\n", enable_video, video_params->localport, video_params->remoteport);
		
		sprintf(argv[0], "%s", "rtp-streamer");
		sprintf(argv[1], "%s", audio_params->remoteaddr);
		sprintf(argv[2], "%d", 1);
		sprintf(argv[3], "%d", audio_params->localport);
		sprintf(argv[4], "%d", audio_params->remoteport);
		sprintf(argv[5], "%d", audio_params->pt);
		sprintf(argv[6], "%d", lc->rtp_conf.audio_jitt_comp);
		sprintf(argv[7], "%d", enable_video);
		sprintf(argv[8], "%d", enable_video? video_params->localport : 0);
		sprintf(argv[9], "%d", enable_video? video_params->remoteport : 0);
		sprintf(argv[10], "%d", enable_video? video_params->pt : 0);
		sprintf(argv[11], "%d", 2000);
		g_warning("Linphone plugin", "child process: loading rtp-streamer..");
		execvp("rtp-streamer", argvv);
		g_warning("Linphone plugin", "child process: what? load failed? errno = %d, %s. \n", errno, strerror(errno));
		_exit(-1);
	}
	else
	{
		close(child_out[1]);
		close(child_in[0]);
		//read(child_out[0], &child_pid, sizeof(pid_t));
		child_pid = pid;
		g_warning("Linphone plugin", "father process has closed 2 descriptors. child pid = %d, ppid = %d\n", child_pid, getpid());
	}
	
}

void
stop_media_stream()
{
		if (child_pid==0)
		{
			g_warning("Linphone plugin", "father process: no child process exists.\n");
			return;
		}
		g_warning("Linphone plugin", "father process is trying to kill chlid process, pid = %d, mypid = %d.\n", child_pid, getpid());
		close(child_in[1]);
		close(child_out[0]);
		kill(child_pid, SIGINT);
		g_warning("Linphone plugin", "father process: reset child_pid to 0\n");
		child_pid = 0;
}
