/***************************************************************************
                          sndutil.h  -  description
                             -------------------
    begin                : Fri May 28 2004
    copyright            : (C) 2004 by Zhanglei Wang
    email                : filamoon@hotmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <linphone_config.h>
#include <sndcard.h>
#include "sndutil.h"
#include <glib.h>
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <mediastream.h>

static int esd_killed=0;
static int artsd_killed=0;

int kill_sound_daemons()
{
	char *command;
	int err;
	FILE *out;
	pid_t pid;

	/* try to kill all artsd*/
	command=g_strdup_printf("ps -u %s |grep artsd",getenv("LOGNAME"));
	out=popen(command,"r");
	if (out!=NULL)
	{
		do{
			err=fscanf(out,"%i",&pid);
			if (err==1) {
				if (linphone_arts_suspend()<0) kill(pid,SIGINT);
				artsd_killed=1;
			}
		}while(err==1);
		pclose(out);
	}
	g_free(command);
	/* kill aplay, because for some reason on kde3 it is spawned when artsd dies*/
	command=g_strdup_printf("ps -u %s |grep aplay",getenv("LOGNAME"));
	out=popen(command,"r");
	if (out!=NULL)
	{
		do{
			err=fscanf(out,"%i",&pid);
			if (err==1) {
				kill(pid,SIGINT);
			}
		}while(err==1);
		pclose(out);
	}
	g_free(command);

	/* do the same with esd*/
	command=g_strdup_printf("ps -u %s |grep esd",getenv("LOGNAME"));
	out=popen(command,"r");
	if (out!=NULL)
	{
		do{
			err=fscanf(out,"%i",&pid);
			if (err==1) {
				kill(pid,SIGINT);
				esd_killed=1;
			}
		}while(err==1);
		pclose(out);
	}
	g_free(command);
	return(0);
}

void restore_sound_daemons()
{
	if (esd_killed){
		esd_killed=0;
		g_message("Restarting esd...");
		g_spawn_command_line_async("esd",NULL);
	}
	if (artsd_killed){
		artsd_killed=0;
		g_message("Restarting artsd...");
		if (linphone_arts_restore()<0) g_spawn_command_line_async("artsd",NULL);
	}
}

/*try to open dsp. If failed, kill sound daemons if allowed by user, and try agin. If fail again, display the error message. Close it in success*/
int try_open_dsp(sound_config_t *sound_conf)
{
	int err1,err2,retval=-1;

	err1=test_audio_dev(sound_conf->dev_id);

	if (err1>=0){
		retval=err1;
		goto end;
	}


	switch (err1){
		case -ENODEV: /* no such device */
		case -ENOENT: /* no such file */
			return err1;
	}

	/* else it is probably EWOULDBLOCK, EAGAIN */
	if (sound_conf->autokill)
	{
		g_message("killing sound daemons.\n");
		kill_sound_daemons();
		sleep(1);   /* wait for sound daemons to be killed...*/
	}
	else
	{
		return(-1);
	}
	err2=test_audio_dev(sound_conf->dev_id);
	if (err2<0)
	{
		printf("Linphone could not open the audio device. Check if your sound card is fully configured and working.");
		return(-1);
	}
	retval=err2;

	end:
	if (retval>0) sound_conf->latency=retval/8;
	return retval;
}


void check_sound_device(SndCard *sndcard)
{
	int fd,len;
	gint a;
	char *file=NULL;
	char *i810_audio=NULL;
	char *snd_pcm_oss=NULL;
	char *snd_mixer_oss=NULL;
	char *snd_pcm=NULL;

	fd=open("/proc/modules",O_RDONLY);
	if (fd>0){
		/* read the entire /proc/modules file and check if sound conf seems correct */
		/*a=fstat(fd,&statbuf);
		if (a<0) g_warning("Can't stat /proc/modules:%s.",strerror(errno));
		len=statbuf.st_size;
		if (len==0) g_warning("/proc/modules has zero size!");
		*/
		/***** fstat does not work on /proc/modules for unknown reason *****/
		len=6000;
		file=g_malloc(len+1);
		a=read(fd,file,len);
		if (a<len) file=g_realloc(file,a+1);
		file[a]='\0';
		i810_audio=strstr(file,"i810_audio");
		if (i810_audio!=NULL){
			printf("You are currently using the i810_audio driver.\nThis driver is buggy and so does not work with Linphone.\nWe suggest that you replace it by its equivalent ALSA driver,\neither with packages from your distribution, or by downloading\nALSA drivers at http://www.alsa-project.org.");
			goto end;
		}
		snd_pcm=strstr(file,"snd-pcm");
		if (snd_pcm!=NULL){
			snd_pcm_oss=strstr(file,"snd-pcm-oss");
			snd_mixer_oss=strstr(file,"snd-mixer-oss");
			if (snd_pcm_oss==NULL){
				printf("Your computer appears to be using ALSA sound drivers.\nThis is the best choice. However the pcm oss emulation module\nis missing and linphone needs it. Please execute\n'modprobe snd-pcm-oss' as root to load it.");
			}
			if (snd_mixer_oss==NULL){
				printf("Your computer appears to be using ALSA sound drivers.\nThis is the best choice. However the mixer oss emulation module\nis missing and linphone needs it. Please execute\n 'modprobe snd-mixer-oss' as root to load it.");
			}
		}
	}else {
#ifdef __LINUX__
		printf("Could not open /proc/modules.");
#endif
	}
	/* now check general volume. Some user forget to rise it and then complain that linphone is
	not working */
	if (sndcard!=NULL){
		a=snd_card_get_level(sndcard,SND_CARD_LEVEL_GENERAL);
		if (a<50){
			g_warning("General level is quite low (%i). Linphone rises it up for you.",a);
			snd_card_set_level(sndcard,SND_CARD_LEVEL_GENERAL,80);
		}
	}
	end:
	if (file!=NULL) g_free(file);
	if (fd>0) close(fd);
}


gint gaim_mediastreamer_get_play_level(sound_config_t * sound_conf)
{
	return sound_conf->play_lev;
}

gint gaim_mediastreamer_get_rec_level(sound_config_t * sound_conf){
	return sound_conf->rec_lev;
}

void gaim_mediastreamer_set_play_level(sound_config_t * sound_conf, gint level){
	sound_conf->play_lev=level;
	g_return_if_fail(sound_conf->sndcard!=NULL);
	snd_card_set_level(sound_conf->sndcard,SND_CARD_LEVEL_OUTPUT,level);
}

void gaim_mediastreamer_set_rec_level(sound_config_t * sound_conf, gint level)
{
	sound_conf->rec_lev=level;
	g_return_if_fail(sound_conf->sndcard!=NULL);
	snd_card_set_level(sound_conf->sndcard,SND_CARD_LEVEL_INPUT,level);
}

gint gaim_mediastreamer_set_sound_device(sound_config_t * sound_conf, gint devid)
{
	SndCard *sndcard;
	int tmp;
	if (devid<0){
		g_warning("Bad devid value: %i",devid);
		devid=0;
	}
	sndcard=snd_card_manager_get_card(snd_card_manager,devid);
	if (sndcard==NULL){
		g_warning("Sound card with index %i does not exist.",devid);
		devid=0;
	}
	sound_conf->dev_id=devid;
	sound_conf->sndcard=sndcard;
	tmp=test_audio_dev(devid);
	if (tmp>0) sound_conf->latency=tmp;
	return 0;
}

gint gaim_mediastreamer_set_sound_device_from_name(sound_config_t * sound_conf,gchar *name)
{
	SndCard *card;
	gint index;
	card=snd_card_manager_get_card_with_string(snd_card_manager,name,&index);
	if (card!=NULL){
		g_message("Setting card with id=%i",index);
		sound_conf->dev_id=index;
		sound_conf->sndcard=card;
		return 0;
	}
	return -1;
}

gint gaim_mediastreamer_get_sound_device(sound_config_t * sound_conf)
{
	return sound_conf->dev_id;
}

SndCardManager * gaim_mediastreamer_get_card_manager(sound_config_t * sound_conf){
	return snd_card_manager;
}

gchar gaim_mediastreamer_get_sound_source(sound_config_t * sound_conf)
{
	return sound_conf->source;
}
void gaim_mediastreamer_set_sound_source(sound_config_t * sound_conf, gchar source)
{
	sound_conf->source=source;
	g_return_if_fail(sound_conf->sndcard!=NULL);
	snd_card_set_rec_source(sound_conf->sndcard,source);
}

void gaim_mediastreamer_set_ring(sound_config_t * sound_conf,gchar *path){
	if (sound_conf->local_ring!=0){
		g_free(sound_conf->local_ring);
	}
	sound_conf->local_ring=g_strdup(path);
}
gchar *gaim_mediastreamer_get_ring(sound_config_t * sound_conf){
	return sound_conf->local_ring;
}
