/*
  The mediastreamer library aims at providing modular media processing and I/O
	for linphone, but also for any telephony application.
  Copyright (C) 2001  Simon MORLAT simon.morlat@linphone.org
  										
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "msavencoder_r263.h"
#include <avcodec.h>

MSFilterInfo AVEncoderR263info={
		"Generic AV encoder",
		0,
		MS_FILTER_VIDEO_CODEC,
		NULL,
		"libr263 wrapper"
};

static MSAVEncoderR263Class *ms_avencoder_r263_class=NULL;
extern char * stream_buffer;
extern int stream_ptr;

MSFilter * ms_AVencoder_r263_new()
{
	MSAVEncoderR263 *enc;
	enc=g_malloc0(sizeof(MSAVEncoderR263));   //modified by wang zhanglei
	if (ms_avencoder_r263_class==NULL)
	{
		 ms_avencoder_r263_class=g_malloc0(sizeof(MSAVEncoderR263Class));
		 ms_AVencoder_r263_class_init(ms_avencoder_r263_class);
	}
	MS_FILTER(enc)->klass=(MSFilterClass*)ms_avencoder_r263_class;
	ms_AVencoder_r263_init(enc);
	return MS_FILTER(enc);
}

void ms_AVencoder_r263_init(MSAVEncoderR263 *enc)
{
	ms_filter_init(MS_FILTER(enc));
	MS_FILTER(enc)->inqueues=enc->q_inputs;
	MS_FILTER(enc)->outqueues=enc->q_outputs;

	enc->pix_fmt=PIX_FMT_YUV420P;	/* YUV420 only */

	/* todo: add an api */
	enc->cparams.format = CPARAM_QCIF;
  enc->width = 176; enc->height = 144;
	
	InitCompress(&enc->cparams);  /* Use standard compression parameters */
	enc->cparams.Q_intra = 8;
	enc->cparams.half_pixel_searchwindow = 4;
	enc->cparams.search_method = CPARAM_LOGARITHMIC;
	enc->cparams.Q_inter = 8;
	enc->cparams.advanced_method = CPARAM_ADVANCED;
	enc->cparams.EncodeThisBlock = enc->EncodeBlocks;
	enc->gop_length = 16;
	enc->cparams.rtp_threshold = 850;
	stream_ptr = 0;
	enc->frame_no = 0;
	enc->msbuf = ms_buffer_new(4096+1024);
	enc->msbuf->ref_count = 1;
	/* stream_buffer = enc->stream_buf; */
	stream_buffer = enc->msbuf->buffer;
}


void ms_AVencoder_r263_class_init(MSAVEncoderR263Class *klass)
{
	ms_filter_class_init(MS_FILTER_CLASS(klass));
	MS_FILTER_CLASS(klass)->max_qinputs=MSAVENCODER_MAX_INPUTS;
	MS_FILTER_CLASS(klass)->max_qoutputs=MSAVENCODER_MAX_INPUTS;
	MS_FILTER_CLASS(klass)->r_maxgran=0;
	MS_FILTER_CLASS(klass)->w_maxgran=0;
	MS_FILTER_CLASS(klass)->destroy=(MSFilterDestroyFunc)ms_AVencoder_r263_destroy;
	MS_FILTER_CLASS(klass)->process=(MSFilterProcessFunc)ms_AVencoder_r263_process;
	ms_filter_class_set_name(MS_FILTER_CLASS(klass),"r263");
}

void ms_AVencoder_r263_uninit(MSAVEncoderR263 *enc)
{
	if (enc->msbuf!=NULL) {
		ms_buffer_destroy(enc->msbuf);
		enc->msbuf=NULL;
	}
	CloseCompress(&enc->cparams);  /* Use standard compression parameters */
}

void ms_AVencoder_r263_destroy( MSAVEncoderR263 *obj)
{
	ms_AVencoder_r263_uninit(obj);
	g_free(obj);
}

__inline__ int _Clip(int x) { if (x<0) return 0; if (x>255) return 255; return x; }

#define SCALEBITS 8
#define ONE_HALF  (1 << (SCALEBITS - 1))
#define FIX(x)		((int) ((x) * (1L<<SCALEBITS) + 0.5))

static void rgb24_to_yuv420p(guint *lum, guint *cb, guint *cr,
                              guint8 *src, int width, int height)
{
    int wrap, wrap3, x, y;
    int r, g, b, r1, g1, b1;
    guint8 *p;

    wrap = width;
    wrap3 = width * 3;
    p = src;
    for(y=0;y<height;y+=2) {
        for(x=0;x<width;x+=2) {
            r = p[0];
            g = p[1];
            b = p[2];
            r1 = r;
            g1 = g;
            b1 = b;
            lum[0] = (FIX(0.29900) * r + FIX(0.58700) * g +
                      FIX(0.11400) * b + ONE_HALF) >> SCALEBITS;
            r = p[3];
            g = p[4];
            b = p[5];
            r1 += r;
            g1 += g;
            b1 += b;
            lum[1] = (FIX(0.29900) * r + FIX(0.58700) * g +
                      FIX(0.11400) * b + ONE_HALF) >> SCALEBITS;
            p += wrap3;
            lum += wrap;

            r = p[0];
            g = p[1];
            b = p[2];
            r1 += r;
            g1 += g;
            b1 += b;
            lum[0] = (FIX(0.29900) * r + FIX(0.58700) * g +
                      FIX(0.11400) * b + ONE_HALF) >> SCALEBITS;
            r = p[3];
            g = p[4];
            b = p[5];
            r1 += r;
            g1 += g;
            b1 += b;
            lum[1] = (FIX(0.29900) * r + FIX(0.58700) * g +
                      FIX(0.11400) * b + ONE_HALF) >> SCALEBITS;

            cb[0] = ((- FIX(0.16874) * r1 - FIX(0.33126) * g1 +
                      FIX(0.50000) * b1 + 4 * ONE_HALF - 1) >> (SCALEBITS + 2)) + 128;
            cr[0] = ((FIX(0.50000) * r1 - FIX(0.41869) * g1 -
                     FIX(0.08131) * b1 + 4 * ONE_HALF - 1) >> (SCALEBITS + 2)) + 128;

            cb++;
            cr++;
            p += -wrap3 + 2 * 3;
            lum += -wrap + 2;
        }
        p += wrap3;
        lum += wrap;
    }
}

static void bgr24_to_yuv420p(guint *lum, guint *cb, guint *cr,
                              guint8 *src, int width, int height)
{
    int wrap, wrap3, x, y;
    int r, g, b, r1, g1, b1;
    guint8 *p;

    wrap = width;
    wrap3 = width * 3;
    p = src;
    for(y=0;y<height;y+=2) {
        for(x=0;x<width;x+=2) {
            r = p[2];
            g = p[1];
            b = p[0];
            r1 = r;
            g1 = g;
            b1 = b;
            lum[0] = (FIX(0.29900) * r + FIX(0.58700) * g +
                      FIX(0.11400) * b + ONE_HALF) >> SCALEBITS;
            r = p[5];
            g = p[4];
            b = p[3];
            r1 += r;
            g1 += g;
            b1 += b;
            lum[1] = (FIX(0.29900) * r + FIX(0.58700) * g +
                      FIX(0.11400) * b + ONE_HALF) >> SCALEBITS;
            p += wrap3;
            lum += wrap;

            r = p[2];
            g = p[1];
            b = p[0];
            r1 += r;
            g1 += g;
            b1 += b;
            lum[0] = (FIX(0.29900) * r + FIX(0.58700) * g +
                      FIX(0.11400) * b + ONE_HALF) >> SCALEBITS;
            r = p[5];
            g = p[4];
            b = p[3];
            r1 += r;
            g1 += g;
            b1 += b;
            lum[1] = (FIX(0.29900) * r + FIX(0.58700) * g +
                      FIX(0.11400) * b + ONE_HALF) >> SCALEBITS;

            cb[0] = ((- FIX(0.16874) * r1 - FIX(0.33126) * g1 +
                      FIX(0.50000) * b1 + 4 * ONE_HALF - 1) >> (SCALEBITS + 2)) + 128;
            cr[0] = ((FIX(0.50000) * r1 - FIX(0.41869) * g1 -
                     FIX(0.08131) * b1 + 4 * ONE_HALF - 1) >> (SCALEBITS + 2)) + 128;

            cb++;
            cr++;
            p += -wrap3 + 2 * 3;
            lum += -wrap + 2;
        }
        p += wrap3;
        lum += wrap;
    }
}

void ms_AVencoder_r263_toggle_rgb_bgr(MSAVEncoderR263 *enc)
{
	if ( enc->input_pix_fmt == PIX_FMT_RGB24)
		enc->input_pix_fmt = PIX_FMT_BGR24;
	else
		enc->input_pix_fmt = PIX_FMT_RGB24;
}
	
void ms_AVencoder_r263_process(MSAVEncoderR263 *r)
{
	MSQueue *inq, *outq;
	MSMessage *inm;
	gint i;
	gint header_int;
	gchar * header = (gchar*) &header_int;
	/*RtpSession * rtp = r->rtp_session; */
	/* guint32 ts; */
	/* gint payloadtype; */
	gchar * buf;
	gint prev_size;

	MSMessage *outm;
	MSBuffer * msbuf = r->msbuf;
	
	inq=r->q_inputs[0];
	outq=r->q_outputs[0];
	
	/* get a picture from the input queue */
	inm=ms_queue_get(inq);
	g_return_if_fail(inm!=NULL);

	/* convert image if necessary */
	if (r->input_pix_fmt == PIX_FMT_RGB24)
		rgb24_to_yuv420p(&(r->qcif_frame.Y[0][0]), &(r->qcif_frame.U[0][0]), &(r->qcif_frame.V[0][0]), (char*)inm->data, r->width, r->height);
	else
		bgr24_to_yuv420p(&(r->qcif_frame.Y[0][0]), &(r->qcif_frame.U[0][0]), &(r->qcif_frame.V[0][0]), (char*)inm->data, r->width, r->height);
	
	
	stream_ptr = 4;
	r->cparams.data = &r->qcif_frame;
	if ( ( (r->frame_no++) % r->gop_length) == 0 )
	{
		r->cparams.inter = CPARAM_INTRA;
		CompressToH263(&r->cparams, &r->bits);
	}
	else
	{
		r->cparams.inter = CPARAM_INTER;
		if(r->cparams.advanced_method)
			FindMotion(&r->cparams,2,2);
		CompressToH263(&r->cparams, &r->bits);
	}
		
	/* payload header */
	header[0] = 0;
	/* header[0] |= (sbit << 3 | ebit); */ /* always byte-aligned */
	//1: SRC(3) I U S A R
	header[1] = 0x40 | ( (r->cparams.inter == CPARAM_INTER) << 4 );   //SRC=010, QCIF
	//2: R R R DBQ(2) TRB(3)
	header[2] = 0;    //R must be 0, we don't use B frames, so DBQ=00, TRB=000
	//3: TR(8)
	header[3] = (char) r->frame_no; //(((int64_t)s->picture_number * 30 * s->avctx->frame_rate_base) /

	/* rtp issues */
	/* ts = (r->sync->time*90000)/1000; */ /* r->frame_no * 9000; /* 25 frames per sec? */

	buf = stream_buffer;
	/* for(i=1; i<r->bits.rtp_count-1; i++) */
	for(i=1; i<r->bits.rtp_count; i++)
	{
			*(int*)buf = header_int;

			outm = ms_message_alloc();
			ms_message_set_buf(outm,msbuf);
			outm->data = buf;
			outm->size = r->bits.rtp_offset[i] - r->bits.rtp_offset[i-1];
			ms_queue_put(outq, outm);
			
			/* rtp_session_send_with_ts(rtp, buf, r->bits.rtp_offset[i] - r->bits.rtp_offset[i-1], ts); */
			buf = stream_buffer + r->bits.rtp_offset[i];
	}
	/* the last packet */
	/*
	*(int*)buf = header_int;
	rtp->payload_type = 34 | 1<<7;
	g_warning("sending rtp packet. frame %d\n", r->frame_no);
	rtp_session_send_with_ts(rtp, buf, r->bits.rtp_offset[r->bits.rtp_count-1] - r->bits.rtp_offset[r->bits.rtp_count-2], ts);
	rtp->payload_type = 34;
	*/
	
	ms_message_destroy(inm);
}

gint ms_AVencoder_r263_set_format(MSAVEncoderR263 *enc, gchar *fmt)
{
	gint format;
	if (strcmp(fmt,"YUV420P")==0) format=PIX_FMT_YUV420P;
	else if (strcmp(fmt,"YUV422")==0) format=PIX_FMT_YUV422;
	else if (strcmp(fmt,"RGB24")==0) format=PIX_FMT_RGB24;	
	else if (strcmp(fmt,"BGR24")==0) format=PIX_FMT_BGR24;
	else if (strcmp(fmt,"YUV422P")==0) format=PIX_FMT_YUV422P;
	else if (strcmp(fmt,"YUV444P")==0) format=PIX_FMT_YUV444P;
	else {
		g_warning("ms_AVdecoder_set_format: unsupported format %s.",fmt);
		return -1;
	}
	enc->input_pix_fmt=format;
	
	return 0;
}

/*
void ms_AVencoder_r263_set_rtp_session(MSAVEncoderR263 *enc, RtpSession * r)
{
	enc->rtp_session = r;
}
*/
