/*
  The mediastreamer library aims at providing modular media processing and I/O
	for linphone, but also for any telephony application.
  Copyright (C) 2001  Simon MORLAT simon.morlat@linphone.org
  										
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "msrtpfastrecv.h"


static MSRtpFastRecvClass *ms_rtp_fast_recv_class=NULL;

MSFilter * ms_rtp_fast_recv_new(void)
{
	MSRtpFastRecv *r;

	r=g_new0(MSRtpFastRecv,1);
	ms_rtp_fast_recv_init(r);
	if (ms_rtp_fast_recv_class==NULL)
	{
		ms_rtp_fast_recv_class=g_new0(MSRtpFastRecvClass,1);
		ms_rtp_fast_recv_class_init(ms_rtp_fast_recv_class);
	}
	MS_FILTER(r)->klass=MS_FILTER_CLASS(ms_rtp_fast_recv_class);
	return(MS_FILTER(r));
}


/* FOR INTERNAL USE*/
void ms_rtp_fast_recv_init(MSRtpFastRecv *r)
{
	ms_filter_init(MS_FILTER(r));
	MS_FILTER(r)->outfifos=r->f_outputs;
	MS_FILTER(r)->outqueues=r->q_outputs;
	memset(r->f_outputs,0,sizeof(MSFifo*)*MSRTPRECV_MAX_OUTPUTS);
	memset(r->q_outputs,0,sizeof(MSFifo*)*MSRTPRECV_MAX_OUTPUTS);
	r->rtpsession=NULL;
	r->stream_started=0;
}

void ms_rtp_fast_recv_class_init(MSRtpFastRecvClass *klass)
{
	ms_filter_class_init(MS_FILTER_CLASS(klass));
	ms_filter_class_set_name(MS_FILTER_CLASS(klass),"RTPFastRecv");
	MS_FILTER_CLASS(klass)->max_qoutputs=MSRTPRECV_MAX_OUTPUTS;
	MS_FILTER_CLASS(klass)->max_foutputs=MSRTPRECV_MAX_OUTPUTS;
	MS_FILTER_CLASS(klass)->w_maxgran=MSRTPRECV_DEF_GRAN;
	ms_filter_class_set_attr(MS_FILTER_CLASS(klass),FILTER_IS_SOURCE);
	MS_FILTER_CLASS(klass)->destroy=(MSFilterDestroyFunc)ms_rtp_fast_recv_destroy;
	MS_FILTER_CLASS(klass)->process=(MSFilterProcessFunc)ms_rtp_fast_recv_process;
	MS_FILTER_CLASS(klass)->setup=(MSFilterSetupFunc)ms_rtp_fast_recv_setup;
}

void ms_rtp_fast_recv_process(MSRtpFastRecv *r)
{
	MSQueue *qo;

	if (r->rtpsession==NULL) return;

  while (rtp_recv_fast(r->rtpsession));
  return;
}

void ms_rtp_fast_recv_destroy( MSRtpFastRecv *obj)
{
	g_free(obj);
}

RtpSession * ms_rtp_fast_recv_set_session(MSRtpFastRecv *obj,RtpSession *session)
{
	RtpSession *old=obj->rtpsession;
	obj->rtpsession=session;
	return old;
}


void ms_rtp_fast_recv_setup(MSRtpFastRecv *r,MSSync *sync)
{
	r->sync=sync;
	r->stream_started=0;
}
