/*
  The mediastreamer library aims at providing modular media processing and I/O
	for linphone, but also for any telephony application.
  Copyright (C) 2001  Simon MORLAT simon.morlat@linphone.org
  										
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef MSRTPFASTRECV_H
#define MSRTPFASTRECV_H

#include "msfilter.h"
#include "mssync.h"

/* because of a conflict between config.h from oRTP and config.h from linphone:*/
#undef PACKAGE
#undef VERSION                                                
#include <ortp.h>

/*this is the class that implements a copy filter*/

#define MSRTPRECV_MAX_OUTPUTS  1 /* max output per filter*/

#define MSRTPRECV_DEF_GRAN 4096 /* the default granularity*/

struct _MSRtpFastRecv
{
    /* the MSCopy derivates from MSFilter, so the MSFilter object MUST be the first of the MSCopy object
       in order to the object mechanism to work*/
    MSFilter filter;
    MSFifo *f_outputs[MSRTPRECV_MAX_OUTPUTS];
	MSQueue *q_outputs[MSRTPRECV_MAX_OUTPUTS];
	MSSync *sync;
	RtpSession *rtpsession;
	gint stream_started;
};

typedef struct _MSRtpFastRecv MSRtpFastRecv;

struct _MSRtpFastRecvClass
{
	/* the MSCopy derivates from MSFilter, so the MSFilter class MUST be the first of the MSCopy class
       in order to the class mechanism to work*/
	MSFilterClass parent_class;
};

typedef struct _MSRtpFastRecvClass MSRtpFastRecvClass;

/* PUBLIC */
#define MS_RTP_FAST_RECV(filter) ((MSRtpFastRecv*)(filter))
#define MS_RTP_FAST_RECV_CLASS(klass) ((MSRtpFastRecvClass*)(klass))
MSFilter * ms_rtp_fast_recv_new(void);
RtpSession * ms_rtp_fast_recv_set_session(MSRtpFastRecv *obj,RtpSession *session);
#define ms_rtp_fast_recv_unset_session(obj) (ms_rtp_fast_recv_set_session((obj),NULL))
#define ms_rtp_fast_recv_get_session(obj) ((obj)->rtpsession)



/* FOR INTERNAL USE*/
void ms_rtp_fast_recv_init(MSRtpFastRecv *r);
void ms_rtp_fast_recv_class_init(MSRtpFastRecvClass *klass);
void ms_rtp_fast_recv_destroy( MSRtpFastRecv *obj);
void ms_rtp_fast_recv_process(MSRtpFastRecv *r);
void ms_rtp_fast_recv_setup(MSRtpFastRecv *r,MSSync *sync);

#endif
