/*
  The mediastreamer library aims at providing modular media processing and I/O
	for linphone, but also for any telephony application.
  Copyright (C) 2001  Simon MORLAT simon.morlat@linphone.org
  										
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "msrtprecv_h263.h"
#include "msrtprecv.h"
#include "msqueue.h"

static MSRtpRecvH263Class *ms_rtp_recv_h263_class=NULL;

MSFilter * ms_rtp_recv_h263_new(void)
{
	MSRtpRecvH263 *r;

	r=g_new0(MSRtpRecvH263,1);
	ms_rtp_recv_h263_init(r);
	if (ms_rtp_recv_h263_class==NULL)
	{
		ms_rtp_recv_h263_class=g_new0(MSRtpRecvH263Class,1);
		ms_rtp_recv_h263_class_init(ms_rtp_recv_h263_class);
	}
	MS_FILTER(r)->klass=MS_FILTER_CLASS(ms_rtp_recv_h263_class);
	return(MS_FILTER(r));
}


/* FOR INTERNAL USE*/
#define REF_IMPOSSIBLE (0xffff + 1)
void ms_rtp_recv_h263_init(MSRtpRecvH263 *r)
{
	ms_rtp_recv_init(MS_RTP_RECV(r));
  r->frame_q = ms_queue_new();
  r->cur_ref = REF_IMPOSSIBLE;
}

void ms_rtp_recv_h263_class_init(MSRtpRecvH263Class *klass)
{
	ms_rtp_recv_class_init(MS_RTP_RECV_CLASS(klass));
	ms_filter_class_set_name(MS_FILTER_CLASS(klass),"RTPRecvH263");
  //functions that are overrided.
	MS_FILTER_CLASS(klass)->destroy=(MSFilterDestroyFunc)ms_rtp_recv_h263_destroy;
	MS_FILTER_CLASS(klass)->process=(MSFilterProcessFunc)ms_rtp_recv_h263_process;
	MS_FILTER_CLASS(klass)->setup=(MSFilterSetupFunc)ms_rtp_recv_h263_setup;
}


static void empty_queue(MSQueue * q)
{
  MSMessage *tmpm = ms_queue_get(q);
  while(tmpm)
  {
      tmpm->buffer->ref_count = 1;
      ms_message_destroy(tmpm);
      tmpm = ms_queue_get(q);
  }
}

static void add_to_queue(MSQueue * q, MSMessage * msg, guint16 idx )
{
  ms_queue_put(q, msg);
  msg->buffer->ref_count = idx;
}

MSMessage * merge_queue(MSQueue *q, guint32 target_ref)
{
   MSMessage * outm=ms_message_new(8192);
   outm->size = 0;
   guint8 prev_pay_hdr = 0;

   //should we reorder the queue first?
   MSMessage * tmpm = ms_queue_get(q);
   while (tmpm)
   {
     guint8 ref = ((guint8*)tmpm->data)[3];
     if (ref != target_ref)
     {   //not in the same frame drop it.
       tmpm->buffer->ref_count = 1;
       ms_message_destroy(tmpm);
       tmpm = ms_queue_get(q);
       continue;
     }

     guint8 pay_hdr = ((guint8*)tmpm->data)[0];
     guint8 ebit = prev_pay_hdr & 0x07;
     guint8 sbit = (pay_hdr & 0x38)>>3;
     int Mode_hdr = (prev_pay_hdr &0x80)==0 ? 4 : 8; //Mode A or Mode B

     if ( ((sbit + ebit) & 7) == 0 ) //matched!
     {
       tmpm->data = ((guint8*)tmpm->data)+Mode_hdr+(sbit>0);
       tmpm->size -= Mode_hdr+(sbit>0);
       memcpy(outm->buffer->buffer + outm->size, tmpm->data, tmpm->size);
       //g_warning("merging packet seq=%d, current seq=%d", seq, hdr->seq_number);
       outm->size += tmpm->size;

       tmpm->buffer->ref_count = 1;
       ms_message_destroy(tmpm);

       tmpm = ms_queue_get(q);
       prev_pay_hdr = pay_hdr;
     }
     else    //missing packet??
     {
       //need some error resillent techniques
       //now we simply drop the whole frame
       g_warning("frame corrupted.");
       empty_queue(q);
       tmpm->buffer->ref_count = 1;
       ms_message_destroy(tmpm);
       ms_message_destroy(outm);
       return NULL;
     }
   }

   return outm;
}

void ms_rtp_recv_h263_process(MSRtpRecvH263 *rh263)
{
	MSFifo *fo;
	MSQueue *qo, *frame_q = rh263->frame_q;
  MSRtpRecv *r = MS_RTP_RECV(rh263);
	//MSSync *sync= r->sync;
	//void *d;
	mblk_t *mp;
	gint len;
	//gint gran=ms_sync_get_samples_per_tick(MS_SYNC(sync));

  MSMessage *msg;
	mblk_t *mdata;
  
	if (r->rtpsession==NULL) return;

  qo=r->q_outputs[0];
	if (qo!=NULL)
	{

#if 0
  while( mp=rtp_session_recvm_without_ts(r->rtpsession) )
  {
    guint8 payload_hdr0 = mp->b_cont->b_rptr[0];
    guint8 payload_hdr1 = mp->b_cont->b_rptr[1];
    guint8 payload_hdr2 = mp->b_cont->b_rptr[2];
    guint8 payload_hdr3 = mp->b_cont->b_rptr[3];
    guint8 prev_pay_hdr = payload_hdr0;
    guint8 sbit, ebit;
    guint8 frame_num = mp->b_cont->b_rptr[3];
    sbit = (payload_hdr0 & 0x38)>>3;
    rtp_header_t * hdr = (rtp_header_t *)mp->b_rptr;

    if (sbit == 0 && hdr->markbit)
    {
    	mdata=mp->b_cont;
    	freeb(mp);
    	msg=msgb_2_ms_message(mdata);
      msg->data = ((guint8*)msg->data)+4;
      msg->size -= 4;
      ms_queue_put(qo, msg);
      return;
    }
      
    msg = ms_message_new(8192);   //we can't imagin a frame larger than 8K...
    int size = mp->b_cont->b_wptr-mp->b_cont->b_rptr-4;
    memcpy(msg->buffer->buffer, mp->b_cont->b_rptr+4, size);
    msg->size = size;
    while(--num)
    {
      mp=rtp_session_recvm_without_ts(r->rtpsession);
      payload_hdr0 = mp->b_cont->b_rptr[0];
      payload_hdr1 = mp->b_cont->b_rptr[1];
      payload_hdr2 = mp->b_cont->b_rptr[2];
      payload_hdr3 = mp->b_cont->b_rptr[3];
      
      ebit = prev_pay_hdr & 0x07;
      sbit = (payload_hdr0 & 0x38)>>3;

      if ( ((sbit + ebit) & 7) == 0 ) //matched!
      {
        size = mp->b_cont->b_wptr - mp->b_cont->b_rptr - 4 - (sbit>0);
        memcpy(msg->buffer->buffer + msg->size, mp->b_cont->b_rptr+4+(sbit>0), size);
        msg->size += size;
        prev_pay_hdr = payload_hdr0;
        freeb(mp->b_cont);
      	freeb(mp);
      }
      else
      {
        ms_message_destroy(msg);
        goto abort;
      }
    }
    ms_queue_put(qo, msg);
    continue;

abort:
    freeb(mp->b_cont);
  	freeb(mp);
    while(mp=rtp_session_recvm_without_ts(r->rtpsession))
    {
      freeb(mp->b_cont);
    	freeb(mp);
    }
}
#endif
    
		while ( (mp=rtp_session_recvm_without_ts(r->rtpsession))!=NULL ){

			guint8 payload_hdr0 = mp->b_cont->b_rptr[0];
			guint8 payload_hdr1 = mp->b_cont->b_rptr[1];
			guint8 payload_hdr2 = mp->b_cont->b_rptr[2];
			guint8 payload_hdr3 = mp->b_cont->b_rptr[3];
			rtp_header_t * hdr = (rtp_header_t *)mp->b_rptr;
			int Mode_hdr = (payload_hdr0 &0x80)==0 ? 4 : 8; //Mode A or Mode B

			//incorrect packet, drop it. 0xc0 = 1100 0000b, test F P bits
			/* we will support Mode A and Mode B
			if (payload_hdr0 & 0xc0)
			{
				freeb(mp->b_cont);
				freeb(mp);
				g_warning("packet dropped directly!");
				return;
			}
			*/

			/* g_warning("packet #%d received, mark=%d", hdr->seq_number, hdr->markbit); */

			/* convert to MSMessage */
			MSMessage *msg;
			mblk_t *mdata;
			r->stream_started=1;
			mdata=mp->b_cont;
			freeb(mp);
			msg=msgb_2_ms_message(mdata);

			if ( hdr->markbit ) //marked frame
			{
				if ( payload_hdr3 == rh263->cur_ref )  //it is what we are expecting
				{
					add_to_queue(frame_q, msg, hdr->seq_number);
					MSMessage * outm = merge_queue(frame_q, payload_hdr3); //this will empty the queue
					if (outm)
					{
						ms_queue_put(qo, outm);
					}
					rh263->cur_ref = REF_IMPOSSIBLE;  //we are not buffering
				}
				else  //not what we are expecting
				{
					if ( (payload_hdr0 &0x38)==0  )  //complete frame. 0x38 = 0011 1000b, test SBIT
					{
						msg->data = ((guint8*)msg->data)+ Mode_hdr; 
						msg->size -= Mode_hdr;
						ms_queue_put(qo, msg);
					}
					else
						ms_message_destroy(msg);

					empty_queue(frame_q);
					rh263->cur_ref = REF_IMPOSSIBLE;  //we are not buffering
				}
				return;
			}
			else //handle unmarked packet
			{
				if (rh263->cur_ref == REF_IMPOSSIBLE) //buffering not started.
				{
					if ( (payload_hdr0 &0x38)!=0 )  //not the 1st fragment, drop it
					{
						ms_message_destroy(msg);
					}
					else  //1st fragment, start buffering
					{
						rh263->cur_ref = payload_hdr3;  //start buffering
						add_to_queue(frame_q, msg, hdr->seq_number);
					}     //end of 1st fragment checking
				}
				else    //buffering has started.
				{
					if (payload_hdr3==rh263->cur_ref)
					{
						add_to_queue(frame_q, msg, hdr->seq_number);
					}
					else
					{
						ms_message_destroy(msg);
					}            
				} //end of buffering checking
			}   //end of unmarked packet
		} //end of while, packet receiving loop
	} //end of if qo
}

void ms_rtp_recv_h263_destroy( MSRtpRecvH263 *obj)
{
	ms_rtp_recv_destroy(MS_RTP_RECV(obj));
  if (obj->frame_q)
  {
    MSMessage * tmp;
    while (tmp = ms_queue_get(obj->frame_q) )
      ms_message_destroy(tmp);
    ms_queue_destroy(obj->frame_q);
  }
}

void ms_rtp_recv_h263_setup(MSRtpRecvH263 *r,MSSync *sync)
{
  ms_rtp_send_setup(MS_RTP_RECV(r), sync);
}
