/*
  The mediastreamer library aims at providing modular media processing and I/O
	for linphone, but also for any telephony application.
  Copyright (C) 2001  Simon MORLAT simon.morlat@linphone.org
  										
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "msrtpsend_h263.h"
#include "mssync.h"
#include "mscodec.h"
#include "msavencoder.h"

static MSRtpSendH263Class *ms_rtp_send_h263_class=NULL;

MSFilter * ms_rtp_send_h263_new(void)
{
	MSRtpSendH263 *r;

	r=g_new0(MSRtpSendH263,1);

	if (ms_rtp_send_h263_class==NULL)
	{
		ms_rtp_send_h263_class=g_new0(MSRtpSendH263Class,1);
		ms_rtp_send_h263_class_init(ms_rtp_send_h263_class);
	}
	MS_FILTER(r)->klass=MS_FILTER_CLASS(ms_rtp_send_h263_class);
	ms_rtp_send_h263_init(r);
	return(MS_FILTER(r));
}


void ms_rtp_send_h263_init(MSRtpSendH263 *r)
{
	ms_rtp_send_init(MS_RTP_SEND(r));
}

void ms_rtp_send_h263_class_init(MSRtpSendH263Class *klass)
{
	ms_rtp_send_class_init(MS_RTP_SEND_CLASS(klass));
	ms_filter_class_set_name(MS_FILTER_CLASS(klass),"RTPSendH263");
  //functions that are overrided.
	MS_FILTER_CLASS(klass)->destroy=(MSFilterDestroyFunc)ms_rtp_send_h263_destroy;
	MS_FILTER_CLASS(klass)->process=(MSFilterProcessFunc)ms_rtp_send_h263_process;
	MS_FILTER_CLASS(klass)->setup=(MSFilterSetupFunc)ms_rtp_send_h263_setup;
}

void ms_rtp_send_h263_process(MSRtpSendH263 *r_h263)
{
  //g_warning("ms_rtp_send_h263_process is called.");

#if 0  
  MSRtpSend * r = MS_RTP_SEND(r_h263);
  MSFifo *fi;
	MSQueue *qi;
	MSSync *sync= r->sync;
	int gran=ms_sync_get_samples_per_tick(sync);
	guint32 ts;
	void *s;
	guint skip;
	guint32 synctime=sync->time;

	g_return_if_fail(gran>0);
	if (r->rtpsession==NULL) return;

	ms_filter_lock(MS_FILTER(r));
	skip=r->delay!=0;
	if (skip) r->delay--;
	/* process output fifo and output queue*/
	fi=r->f_inputs[0];
	if (fi!=NULL)
	{
		ts=get_new_timestamp(r,synctime);
		/* try to read r->packet_size bytes and send them in a rtp packet*/
		ms_fifo_get_read_ptr(fi,r->packet_size,&s);
		if (!skip){
			rtp_session_send_with_ts(r->rtpsession,s,r->packet_size,ts);
			ms_trace("len=%i, ts=%i ",r->packet_size,ts);
		}
	}
	qi=r->q_inputs[0];
	if (qi!=NULL)
	{
		MSMessage *msg;
		/* read a MSMessage and send it through the network*/
		while ( (msg=ms_queue_get(qi))!=NULL){
      
			ts=get_new_timestamp(r,synctime);
			if (!skip) {
				rtp_session_send_with_ts(r->rtpsession,msg->data,msg->size,ts);

			}
			ms_message_destroy(msg);
		}
	}
	ms_filter_unlock(MS_FILTER(r));
#endif
}

void ms_rtp_send_h263_destroy( MSRtpSendH263 *obj)
{
	ms_rtp_send_destroy(MS_RTP_SEND(obj));
}

void ms_rtp_send_h263_setup(MSRtpSendH263 *r, MSSync *sync)
{
  ms_rtp_send_setup(MS_RTP_SEND(r), sync);
}

/* this function is called by ffmpeg codec later than 0.4.6 */
/* since packet_number is not used by ffmpeg, we use it to pass h263 payload header */
static void rtp_callback(void *data, int size, int payload_header, int bMarkbit, void * priv_data)
{
  //g_warning("rtp_callback is called with data=%d,size=%d, payload_header=%d, Markbit=%d", (int)data, size, payload_header, bMarkbit);

	MSRtpSend * r = MS_RTP_SEND(priv_data);
	MSQueue *qi;
	MSSync *sync= r->sync;
	gint payloadtype;
  
	gchar * buffer = (gchar*)data - 4;      //prepend 4 bytes of h.263 payload header type A, RFC 2190
        	                                //remember we have reserved 4 bytes at the beginning of the buffer at msavencoder.c
	*(int*)buffer = payload_header;

	int gran=ms_sync_get_samples_per_tick(sync);
	guint32 ts;
	void *s;
	guint skip;
	guint32 synctime=sync->time;

	g_return_if_fail(gran>0);
	if (r->rtpsession==NULL) return;

 	ts=get_new_timestamp(r,synctime);

	payloadtype = r->rtpsession->payload_type;
	r->rtpsession->payload_type |= bMarkbit << 7;
	rtp_session_send_with_ts(r->rtpsession,buffer,size+4,ts);
	r->rtpsession->payload_type = payloadtype;
}

void ms_rtp_send_h263_set_codec(MSRtpSendH263 *r, MSAVEncoder * codec)
{
  codec->av_context.rtp_callback = rtp_callback;
  codec->av_context.rtp_payload_size = 850;    //MTU is 1500, ip header, rtp header(12 bytes), h.263 payload header(4 bytes)
  codec->av_context.opaque = r;
}



